@@ -71,20 +71,20 @@
 
 #include <libwnck/libwnck.h>
 
-#include "xfsm-shutdown-helper.h"
 #include <libxfce4ui/libxfce4ui.h>
 
 #include <libxfsm/xfsm-splash-engine.h>
 #include <libxfsm/xfsm-util.h>
 
-#include "xfsm-manager.h"
-#include "chooser-icon.h"
-#include "xfsm-chooser.h"
-#include "xfsm-global.h"
-#include "xfsm-legacy.h"
-#include "xfsm-startup.h"
-#include "xfsm-marshal.h"
-#include "xfsm-error.h"
+#include <xfce4-session/xfsm-manager.h>
+#include <xfce4-session/xfsm-chooser-icon.h>
+#include <xfce4-session/xfsm-chooser.h>
+#include <xfce4-session/xfsm-global.h>
+#include <xfce4-session/xfsm-legacy.h>
+#include <xfce4-session/xfsm-startup.h>
+#include <xfce4-session/xfsm-marshal.h>
+#include <xfce4-session/xfsm-error.h>
+#include <xfce4-session/xfsm-logout-dialog.h>
 
 
 #define DEFAULT_SESSION_NAME "Default"
@@ -95,13 +95,17 @@ struct _XfsmManager
   GObject parent;
 
   XfsmManagerState state;
-  XfsmShutdownType shutdown_type;
+
+  XfsmShutdownType  shutdown_type;
+  XfsmShutdown     *shutdown_helper;
 
   gboolean         session_chooser;
   gchar           *session_name;
   gchar           *session_file;
   gchar           *checkpoint_session_name;
 
+  gboolean         start_at;
+
   gboolean         compat_gnome;
   gboolean         compat_kde;
 
@@ -225,6 +229,7 @@ xfsm_manager_init (XfsmManager *manager)
   manager->session_chooser = FALSE;
   manager->failsafe_mode = TRUE;
   manager->shutdown_type = XFSM_SHUTDOWN_LOGOUT;
+  manager->shutdown_helper = xfsm_shutdown_get ();
 
   manager->pending_properties = g_queue_new ();
   manager->starting_properties = g_queue_new ();
@@ -243,6 +248,8 @@ xfsm_manager_finalize (GObject *obj)
   if (manager->die_timeout_id != 0)
     g_source_remove (manager->die_timeout_id);
 
+  g_object_unref (manager->shutdown_helper);
+
   g_queue_foreach (manager->pending_properties, (GFunc) xfsm_properties_free, NULL);
   g_queue_free (manager->pending_properties);
 
@@ -478,7 +485,7 @@ xfsm_manager_choose_session (XfsmManager *manager,
             {
               if (G_UNLIKELY (preview_default == NULL))
                 {
-                  preview_default = gdk_pixbuf_new_from_inline (-1, chooser_icon_data,
+                  preview_default = gdk_pixbuf_new_from_inline (-1, xfsm_chooser_icon_data,
                                                                 FALSE, NULL);
                 }
 
@@ -729,6 +736,7 @@ xfsm_manager_load (XfsmManager   *manager,
 
   manager->compat_gnome = xfconf_channel_get_bool (channel, "/compat/LaunchGNOME", FALSE);
   manager->compat_kde = xfconf_channel_get_bool (channel, "/compat/LaunchKDE", FALSE);
+  manager->start_at = xfconf_channel_get_bool (channel, "/general/StartAssistiveTechnologies", FALSE);
 
   display_name  = xfsm_gdk_display_get_fullname (gdk_display_get_default ());
 
@@ -763,7 +771,7 @@ xfsm_manager_restart (XfsmManager *manager)
   /* tell splash screen that the session is starting now */
   preview = xfsm_load_session_preview (manager->session_name);
   if (preview == NULL)
-    preview = gdk_pixbuf_new_from_inline (-1, chooser_icon_data, FALSE, NULL);
+    preview = gdk_pixbuf_new_from_inline (-1, xfsm_chooser_icon_data, FALSE, NULL);
   steps = g_queue_get_length (manager->failsafe_mode ? manager->failsafe_clients : manager->pending_properties);
   xfsm_splash_screen_start (splash_screen, manager->session_name, preview, steps);
   g_object_unref (preview);
@@ -1081,8 +1089,9 @@ xfsm_manager_save_yourself_global (XfsmManager     *manager,
                                    XfsmShutdownType shutdown_type,
                                    gboolean         allow_shutdown_save)
 {
-  gboolean shutdown_save = allow_shutdown_save;
-  GList *lp;
+  gboolean  shutdown_save = allow_shutdown_save;
+  GList    *lp;
+  GError   *error = NULL;
 
   if (shutdown)
     {
@@ -1090,7 +1099,7 @@ xfsm_manager_save_yourself_global (XfsmManager     *manager,
         {
           /* if we're not specifying fast shutdown, and we're ok with
            * prompting then ask the user what to do */
-          if (!shutdownDialog (manager->session_name, &manager->shutdown_type, &shutdown_save))
+          if (!xfsm_logout_dialog (manager->session_name, &manager->shutdown_type, &shutdown_save))
             return;
 
           /* |allow_shutdown_save| is ignored if we prompt the user.  i think
@@ -1107,14 +1116,9 @@ xfsm_manager_save_yourself_global (XfsmManager     *manager,
       if (manager->shutdown_type == XFSM_SHUTDOWN_SUSPEND
           || manager->shutdown_type == XFSM_SHUTDOWN_HIBERNATE)
         {
-          XfsmShutdownHelper *shutdown_helper;
-          GError *error = NULL;
-
-          shutdown_helper = xfsm_shutdown_helper_new ();
-
-          if (!xfsm_shutdown_helper_send_command (shutdown_helper,
-                                                  manager->shutdown_type,
-                                                  &error))
+          if (!xfsm_shutdown_try_type (manager->shutdown_helper,
+                                       manager->shutdown_type,
+                                       &error))
             {
               xfce_message_dialog (NULL, _("Shutdown Failed"),
                                    GTK_STOCK_DIALOG_ERROR,
@@ -1127,10 +1131,6 @@ xfsm_manager_save_yourself_global (XfsmManager     *manager,
               g_error_free (error);
             }
 
-          /* clean up and return */
-          g_object_unref (shutdown_helper);
-
-
           /* at this point, either we failed to suspend/hibernate, or we
            * successfully suspended/hibernated, and we've been woken back
            * up, so return control to the user */
@@ -1211,6 +1211,8 @@ xfsm_manager_save_yourself (XfsmManager *manager,
 {
   if (G_UNLIKELY (xfsm_client_get_state (client) != XFSM_CLIENT_IDLE))
     {
+
+
       xfsm_verbose ("Client Id = %s, requested SAVE YOURSELF, but client is not in IDLE mode.\n"
                     "   Client will be nuked now.\n\n",
                     xfsm_client_get_id (client));
@@ -1774,6 +1776,12 @@ xfsm_manager_get_compat_startup (XfsmManager          *manager,
 }
 
 
+gboolean
+xfsm_manager_get_start_at (XfsmManager *manager)
+{
+  return manager->start_at;
+}
+
 
 /*
  * dbus server impl
@@ -1797,14 +1805,36 @@ static gboolean xfsm_manager_dbus_get_state (XfsmManager *manager,
 static gboolean xfsm_manager_dbus_checkpoint (XfsmManager *manager,
                                               const gchar *session_name,
                                               GError     **error);
+static gboolean xfsm_manager_dbus_logout (XfsmManager *manager,
+                                          gboolean     show_dialog,
+                                          gboolean     allow_save,
+                                          GError     **error);
 static gboolean xfsm_manager_dbus_shutdown (XfsmManager *manager,
-                                            guint        type,
                                             gboolean     allow_save,
                                             GError     **error);
+static gboolean xfsm_manager_dbus_can_shutdown (XfsmManager *manager,
+                                                gboolean    *can_shutdown,
+                                                GError     **error);
+static gboolean xfsm_manager_dbus_restart (XfsmManager *manager,
+                                           gboolean     allow_save,
+                                           GError     **error);
+static gboolean xfsm_manager_dbus_can_restart (XfsmManager *manager,
+                                               gboolean    *can_restart,
+                                               GError     **error);
+static gboolean xfsm_manager_dbus_suspend (XfsmManager *manager,
+                                           GError     **error);
+static gboolean xfsm_manager_dbus_can_suspend (XfsmManager *manager,
+                                               gboolean    *can_suspend,
+                                               GError     **error);
+static gboolean xfsm_manager_dbus_hibernate (XfsmManager *manager,
+                                             GError     **error);
+static gboolean xfsm_manager_dbus_can_hibernate (XfsmManager *manager,
+                                                 gboolean    *can_hibernate,
+                                                 GError     **error);
 
 
 /* eader needs the above fwd decls */
-#include "xfsm-manager-dbus.h"
+#include <xfce4-session/xfsm-manager-dbus.h>
 
 
 static void
@@ -1974,10 +2004,10 @@ xfsm_manager_dbus_shutdown_idled (gpointer data)
 
 
 static gboolean
-xfsm_manager_dbus_shutdown (XfsmManager *manager,
-                            guint        type,
-                            gboolean     allow_save,
-                            GError     **error)
+xfsm_manager_save_yourself_dbus (XfsmManager       *manager,
+                                 XfsmShutdownType   type,
+                                 gboolean           allow_save,
+                                 GError           **error)
 {
   ShutdownIdleData *idata;
 
@@ -1988,13 +2018,6 @@ xfsm_manager_dbus_shutdown (XfsmManager *manager,
       return FALSE;
     }
 
-  if (type > XFSM_SHUTDOWN_HIBERNATE)
-    {
-      g_set_error (error, XFSM_ERROR, XFSM_ERROR_BAD_VALUE,
-                   _("Invalid shutdown type \"%u\""), type);
-      return FALSE;
-    }
-
   idata = g_new (ShutdownIdleData, 1);
   idata->manager = manager;
   idata->type = type;
@@ -2004,3 +2027,118 @@ xfsm_manager_dbus_shutdown (XfsmManager *manager,
 
   return TRUE;
 }
+
+
+static gboolean
+xfsm_manager_dbus_logout (XfsmManager *manager,
+                          gboolean     show_dialog,
+                          gboolean     allow_save,
+                          GError     **error)
+{
+  XfsmShutdownType type;
+
+  g_return_val_if_fail (XFSM_IS_MANAGER (manager), FALSE);
+
+  type = show_dialog ? XFSM_SHUTDOWN_ASK : XFSM_SHUTDOWN_LOGOUT;
+  return xfsm_manager_save_yourself_dbus (manager, type,
+                                          allow_save, error);
+}
+
+
+static gboolean
+xfsm_manager_dbus_shutdown (XfsmManager *manager,
+                            gboolean     allow_save,
+                            GError     **error)
+{
+  g_return_val_if_fail (XFSM_IS_MANAGER (manager), FALSE);
+  return xfsm_manager_save_yourself_dbus (manager, XFSM_SHUTDOWN_SHUTDOWN,
+                                          allow_save, error);
+}
+
+
+static gboolean
+xfsm_manager_dbus_can_shutdown (XfsmManager *manager,
+                                gboolean    *can_shutdown,
+                                GError     **error)
+{
+  g_return_val_if_fail (XFSM_IS_MANAGER (manager), FALSE);
+  return xfsm_shutdown_can_shutdown (manager->shutdown_helper,
+                                     can_shutdown, error);
+}
+
+
+static gboolean
+xfsm_manager_dbus_restart (XfsmManager *manager,
+                           gboolean     allow_save,
+                           GError     **error)
+{
+  g_return_val_if_fail (XFSM_IS_MANAGER (manager), FALSE);
+  return xfsm_manager_save_yourself_dbus (manager, XFSM_SHUTDOWN_RESTART,
+                                          allow_save, error);
+}
+
+
+static gboolean
+xfsm_manager_dbus_can_restart (XfsmManager *manager,
+                               gboolean    *can_restart,
+                               GError     **error)
+{
+  g_return_val_if_fail (XFSM_IS_MANAGER (manager), FALSE);
+  return xfsm_shutdown_can_restart (manager->shutdown_helper,
+                                    can_restart, error);
+}
+
+
+static gboolean
+xfsm_manager_dbus_suspend (XfsmManager *manager,
+                           GError     **error)
+{
+  g_return_val_if_fail (XFSM_IS_MANAGER (manager), FALSE);
+  return xfsm_shutdown_try_suspend (manager->shutdown_helper, error);
+}
+
+
+static gboolean
+xfsm_manager_dbus_can_suspend (XfsmManager *manager,
+                               gboolean    *can_suspend,
+                               GError     **error)
+{
+  gboolean retval;
+  gboolean auth_suspend;
+
+  g_return_val_if_fail (XFSM_IS_MANAGER (manager), FALSE);
+  retval = xfsm_shutdown_can_suspend (manager->shutdown_helper,
+                                      can_suspend, &auth_suspend, error);
+
+  if (!auth_suspend)
+    can_suspend = FALSE;
+
+  return retval;
+}
+
+static gboolean
+xfsm_manager_dbus_hibernate (XfsmManager *manager,
+                             GError     **error)
+{
+  g_return_val_if_fail (XFSM_IS_MANAGER (manager), FALSE);
+  return xfsm_shutdown_try_hibernate (manager->shutdown_helper, error);
+}
+
+
+static gboolean
+xfsm_manager_dbus_can_hibernate (XfsmManager *manager,
+                                 gboolean    *can_hibernate,
+                                 GError     **error)
+{
+  gboolean retval;
+  gboolean auth_hibernate;
+
+  g_return_val_if_fail (XFSM_IS_MANAGER (manager), FALSE);
+  retval = xfsm_shutdown_can_hibernate (manager->shutdown_helper,
+                                        can_hibernate, &auth_hibernate, error);
+
+  if (!auth_hibernate)
+    can_hibernate = FALSE;
+
+  return retval;
+}
