@@ -45,7 +45,7 @@
 #endif
 
 #include <glib/gstdio.h>
-
+#include <gdk/gdkx.h>
 #include <libxfce4ui/libxfce4ui.h>
 
 #include <libxfsm/xfsm-util.h>
@@ -56,7 +56,7 @@
 #include <xfce4-session/xfsm-manager.h>
 #include <xfce4-session/xfsm-splash-screen.h>
 
-#include "xfsm-startup.h"
+#include <xfce4-session/xfsm-startup.h>
 
 
 typedef struct
@@ -303,15 +303,14 @@ xfsm_startup_autostart_migrate (void)
 
 
 static gint
-xfsm_startup_autostart_xdg (XfsmManager *manager)
+xfsm_startup_autostart_xdg (XfsmManager *manager,
+                            gboolean     start_at_spi)
 {
   const gchar *try_exec;
   const gchar *type;
   const gchar *exec;
   gboolean     startup_notify;
   gboolean     terminal;
-  gboolean     gnome;
-  gboolean     kde;
   gboolean     skip;
   GError      *error = NULL;
   XfceRc      *rc;
@@ -320,15 +319,19 @@ xfsm_startup_autostart_xdg (XfsmManager *manager)
   gchar      **not_show_in;
   gint         started = 0;
   gint         n, m;
+  gchar       *filename;
+  const gchar *pattern;
 
   /* migrate the old autostart location (if still present) */
   xfsm_startup_autostart_migrate ();
 
-  /* Get if we should start KDE and GNOME applications */
-  kde = xfsm_manager_get_compat_startup (manager, XFSM_MANAGER_COMPAT_KDE);
-  gnome = xfsm_manager_get_compat_startup (manager, XFSM_MANAGER_COMPAT_GNOME);
+  /* pattern for only at-spi desktop files or everything */
+  if (start_at_spi)
+    pattern = "autostart/at-spi-*.desktop";
+  else
+    pattern = "autostart/*.desktop";
 
-  files = xfce_resource_match (XFCE_RESOURCE_CONFIG, "autostart/*.desktop", TRUE);
+  files = xfce_resource_match (XFCE_RESOURCE_CONFIG, pattern, TRUE);
   for (n = 0; files[n] != NULL; ++n)
     {
       rc = xfce_rc_config_open (XFCE_RESOURCE_CONFIG, files[n], TRUE);
@@ -341,45 +344,53 @@ xfsm_startup_autostart_xdg (XfsmManager *manager)
       skip = xfce_rc_read_bool_entry (rc, "Hidden", FALSE);
       if (G_LIKELY (!skip))
         {
-          /* check the OnlyShowIn setting */
-          only_show_in = xfce_rc_read_list_entry (rc, "OnlyShowIn", ";");
-          if (G_UNLIKELY (only_show_in != NULL))
+          if (xfce_rc_read_bool_entry (rc, "X-XFCE-Autostart-Override", FALSE))
             {
-              /* check if "XFCE" is specified */
-              /* If we start the GNOME components, also start the applications when
-               * then "GNOME" is specified. */
-              /* If we start the KDE components, also start the applications when
-               * then "KDE" is specified. */
-              for (m = 0, skip = TRUE; only_show_in[m] != NULL; ++m)
-                {
-                  if ((g_ascii_strcasecmp (only_show_in[m], "XFCE") == 0) ||
-                      (gnome && g_ascii_strcasecmp (only_show_in[m], "GNOME") == 0) ||
-                      (kde && g_ascii_strcasecmp (only_show_in[m], "KDE") == 0))
-                    {
-                      skip = FALSE;
-                      break;
-                    }
-                }
-
-              g_strfreev (only_show_in);
+              /* override the OnlyShowIn check */
+              skip = FALSE;
             }
           else
             {
-              /* check the NotShowIn setting */
-              not_show_in = xfce_rc_read_list_entry (rc, "NotShowIn", ";");
-              if (G_UNLIKELY (not_show_in != NULL))
+              /* check the OnlyShowIn setting */
+              only_show_in = xfce_rc_read_list_entry (rc, "OnlyShowIn", ";");
+              if (G_UNLIKELY (only_show_in != NULL))
                 {
-                  /* check if "Xfce" is not specified */
-                  for (m = 0; not_show_in[m] != NULL; ++m)
-                    if (g_ascii_strcasecmp (not_show_in[m], "Xfce") == 0)
-                      {
-                        skip = TRUE;
-                        break;
-                      }
+                  /* check if "XFCE" is specified */
+                  for (m = 0, skip = TRUE; only_show_in[m] != NULL; ++m)
+                    {
+                      if (g_ascii_strcasecmp (only_show_in[m], "XFCE") == 0)
+                        {
+                          skip = FALSE;
+                          break;
+                        }
+                    }
 
-                  g_strfreev (not_show_in);
+                  g_strfreev (only_show_in);
                 }
             }
+
+          /* check the NotShowIn setting */
+          not_show_in = xfce_rc_read_list_entry (rc, "NotShowIn", ";");
+          if (G_UNLIKELY (not_show_in != NULL))
+            {
+              /* check if "Xfce" is not specified */
+              for (m = 0; not_show_in[m] != NULL; ++m)
+                if (g_ascii_strcasecmp (not_show_in[m], "XFCE") == 0)
+                  {
+                    skip = TRUE;
+                    break;
+                  }
+
+              g_strfreev (not_show_in);
+            }
+
+          /* skip at-spi launchers if not in at-spi mode or don't skip
+           * them no matter what the OnlyShowIn key says if only
+           * launching at-spi */
+          filename = g_path_get_basename (files[n]);
+          if (g_str_has_prefix (filename, "at-spi-"))
+            skip = !start_at_spi;
+          g_free (filename);
         }
 
       /* check the "Type" key */
@@ -433,7 +444,7 @@ xfsm_startup_autostart (XfsmManager *manager)
 {
   gint n;
 
-  n = xfsm_startup_autostart_xdg (manager);
+  n = xfsm_startup_autostart_xdg (manager, FALSE);
 
   if (n > 0)
     {
@@ -461,9 +472,117 @@ xfsm_startup_foreign (XfsmManager *manager)
 }
 
 
+static void
+xfsm_startup_at_set_gtk_modules (void)
+{
+  const gchar  *old;
+  gchar       **modules;
+  guint         i;
+  gboolean      found_gail = FALSE;
+  gboolean      found_atk_bridge = FALSE;
+  GString      *new;
+
+  old = g_getenv ("GTK_MODULES");
+  if (old != NULL && *old != '\0')
+    {
+      /* check which modules are already loaded */
+      modules = g_strsplit (old, ":", -1);
+      for (i = 0; modules[i] != NULL; i++)
+        {
+          if (strcmp (modules[i], "gail") == 0)
+            found_gail = TRUE;
+          else if (strcmp (modules[i], "atk-bridge") == 0)
+            found_atk_bridge = TRUE;
+        }
+      g_strfreev (modules);
+
+      if (!found_gail || !found_atk_bridge)
+        {
+          /* append modules to old value */
+          new = g_string_new (old);
+          if (!found_gail)
+            new = g_string_append (new, ":gail");
+          if (!found_atk_bridge)
+            new = g_string_append (new, ":atk-bridge");
+
+          g_setenv ("GTK_MODULES", new->str, TRUE);
+          g_string_free (new, TRUE);
+        }
+    }
+  else
+    {
+      g_setenv ("GTK_MODULES", "gail:atk-bridge", TRUE);
+    }
+}
+
+
+static gboolean
+xfsm_startup_at_spi_ior_set (void)
+{
+  Atom        AT_SPI_IOR;
+  GdkDisplay *display;
+  Atom        actual_type;
+  gint        actual_format;
+  guchar    *data = NULL;
+  gulong     nitems;
+  gulong     leftover;
+
+  display = gdk_display_get_default ();
+  AT_SPI_IOR = XInternAtom (GDK_DISPLAY_XDISPLAY (display), "AT_SPI_IOR", False);
+  XGetWindowProperty (GDK_DISPLAY_XDISPLAY (display),
+                      XDefaultRootWindow (GDK_DISPLAY_XDISPLAY (display)),
+                      AT_SPI_IOR, 0L,
+                      (long) BUFSIZ, False,
+                      (Atom) 31, &actual_type, &actual_format,
+                      &nitems, &leftover, &data);
+
+  if (data == NULL)
+    return FALSE;
+  XFree (data);
+
+  return TRUE;
+}
+
+
+static void
+xfsm_startup_at (XfsmManager *manager)
+{
+  gint n, i;
+
+  /* start at-spi-dbus-bus and/or at-spi-registryd */
+  n = xfsm_startup_autostart_xdg (manager, TRUE);
+
+  if (n > 0)
+    {
+      if (G_LIKELY (splash_screen != NULL))
+        xfsm_splash_screen_next (splash_screen, _("Starting Assistive Technologies"));
+
+      xfsm_startup_at_set_gtk_modules ();
+
+      /* wait for 2 seconds until the at-spi registered, not very nice
+       * but required to properly start an accessible desktop */
+      for (i = 0; i < 10; i++)
+        {
+          if (xfsm_startup_at_spi_ior_set ())
+            break;
+          xfsm_verbose ("Waiting for at-spi to register...\n");
+          g_usleep (G_USEC_PER_SEC / 5);
+        }
+    }
+  else
+    {
+      g_warning ("No assistive technology service provider was started!");
+    }
+}
+
+
 void
 xfsm_startup_begin (XfsmManager *manager)
 {
+  /* start assistive technology before anything else */
+  if (xfsm_manager_get_start_at (manager))
+    xfsm_startup_at (manager);
+
   if (xfsm_manager_get_use_failsafe_mode (manager))
     {
       xfsm_startup_failsafe (manager);
